# Copyright (c) 2008 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import re

def strip_chars(string, chars):
    """ strip_chars(string, chars) -> string
    
    Remove all characters listed in chars and return the resulting string.
    
    """
    
    for character in chars:
        string = string.replace(character, '')
    
    return string

def is_format_package_name(string):
    """ is_format_package_name(string) -> bool
    
    Return True if string would be an acceptable name for a Debian package.
    
    """
    
    return strip_chars(string, '+-.').isalnum() and string.islower() and \
        string[0].isalnum() and len(string) > 1

def is_format_user_name(string):
    """ is_format_user_name(string) -> bool
    
    Return True if string would be an acceptable username (that is, if all
    characters that it includes are allowed in an e-mail address).
    
    """
    
    sanitized = strip_chars(string, '@.-_')
    
    # We need the second condition so that strings which only contain valid
    # characters (but where none of those is alphanumeric) give True.
    # This is necessary for the sanitize_username function, which will pass
    # individual characters, one for one, to this function.
    return sanitized.isalnum() or not sanitized

def sanitize_username(username):
    """ strip_chars(username) -> string
    
    Remove all characters from the given string which are not allowed in
    an username.
    
    """
    
    return filter(is_format_user_name, username)

def display_email(email):
    """ display_email(email) -> string
    
    Return the given address after obfuscating it for some minimal
    protection against e-mail harvesting.
    
    """
    
    return email.replace('@', '&#64;') # <span class="hidden">(REMOVE THIS)</span>

def link_bugs(string):
    """ link_bugs(string) -> string
    
    Searches for bug numbers in a string and converts those into links to
    Launchpad's Malone.
    
    """
    
    make_link = lambda x: '<a href="http://launchpad.net/bugs/%s">%s</a>' % \
            (''.join([n for n in x.group() if n.isdigit()]), x.group())
    
    return re.sub('(?:[bB][uU][gG]|LP):?\s+?#?([0-9]+)', make_link, string)

def to_list(obj):
    """ to_list(obj) -> list
    
    Take a list or a tuple (if it's the later one, convert it to a list), run
    through all of it's elements and if they are tuples convert them to lists
    and finally return the resulting tuple.
    
    """
    
    if type(obj) is not list:
        obj = list(obj)
    
    for x in range(0, len(obj)):
        if type(obj[x]) is tuple:
            obj[x] = list(obj[x])
    
    return obj
